package com.woosim.sample;

import android.Manifest;
import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.support.annotation.NonNull;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.SimpleAdapter;
import android.widget.Toast;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;



public class MainActivity extends AppCompatActivity {
    // Debugging
    private static final String TAG = "MainActivity";
    private static final boolean D = true;

    // Message type & name received from the BluetoothPrintService handler
    protected static final int MESSAGE_DEVICE_NAME = 1;
    protected static final int MESSAGE_TOAST = 2;
    protected static final int MESSAGE_READ = 3;
    protected static final String DEVICE_NAME = "device_name";
    protected static final String TOAST = "toast";

    // Intent request codes
    private static final int REQUEST_CONNECT_DEVICE_SECURE = 1;
    private static final int REQUEST_CONNECT_DEVICE_INSECURE = 2;
    private static final int REQUEST_ENABLE_BT = 3;

    private static final int PERMISSION_DEVICE_SCAN_SECURE = 11;
    private static final int PERMISSION_DEVICE_SCAN_INSECURE = 12;

    private Menu mMenu = null;
    private BluetoothAdapter mBluetoothAdapter = null;	// Local Bluetooth adapter
    protected static BluetoothPrintService mPrintService = null;	// Member object for the print services


    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        if(D) Log.i(TAG, "+++ ON CREATE +++");

        // Get local Bluetooth adapter
        mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();

        // If the adapter is null, then Bluetooth is not supported
        if (mBluetoothAdapter == null) {
            Toast.makeText(this, R.string.bt_na, Toast.LENGTH_LONG).show();
            finish();
            return;
        }

        // Make a list for sample activities
        final ListView simpleListView= findViewById(R.id.simpleListView);

        SimpleAdapter adapter = new SimpleAdapter(this,
                getData(),
                android.R.layout.simple_list_item_1,
                new String[] { "title" },
                new int[] { android.R.id.text1 });

        if (simpleListView != null) {
            simpleListView.setAdapter(adapter);
            //perform listView item click event
            simpleListView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
                @Override
                public void onItemClick(AdapterView<?> adapterView, View view, int position, long l) {
                    Map map = (Map) simpleListView.getItemAtPosition(position);
                    Intent intent = (Intent) map.get("intent");
                    startActivity(intent);
                }
            });
        }
    }

    private ArrayList<HashMap<String, Object>> getData() {
        ArrayList<HashMap<String, Object>> appList = new ArrayList<>();

        addItem(appList, getString(R.string.title_sign_pad), new Intent(this, SignPad.class));
        addItem(appList, getString(R.string.title_multi_language), new Intent(this, MultiLanguage.class));
        addItem(appList, getString(R.string.title_pagemode), new Intent(this, PageMode.class));

        return appList;
    }

    private void addItem(ArrayList<HashMap<String, Object>> list, String name, Intent intent) {
        HashMap<String, Object> temp = new HashMap<>();

        temp.put("title", name);
        temp.put("intent", intent);
        list.add(temp);
    }

    @Override
    public void onStart() {
        super.onStart();
        if(D) Log.i(TAG, "++ ON START ++");

        if (!mBluetoothAdapter.isEnabled()) {
            // Request to enable bluetooth.
            // Bluetooth session will then be setup during onActivityResult
            Intent intent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
            startActivityForResult(intent, REQUEST_ENABLE_BT);
        } else {
            if (mPrintService == null) {
                // Initialize the BluetoothPrintService to perform bluetooth connections
                mPrintService = new BluetoothPrintService(mHandler);
            }
        }
    }

    // The handler that gets information back from the BluetoothPrintService
    private final MyHandler mHandler = new MyHandler(this);

    private static class MyHandler extends Handler {
        private final WeakReference<MainActivity> mActivity;

        MyHandler(MainActivity activity) {
            mActivity = new WeakReference<>(activity);
        }

        @Override
        public void handleMessage(Message msg) {
            MainActivity activity = mActivity.get();
            if (activity != null) {
                activity.handleMessage(msg);
            }
        }
    }

    private void handleMessage(Message msg) {
        switch (msg.what) {
        case MESSAGE_DEVICE_NAME:
            // save the connected device's name
            String deviceName = msg.getData().getString(DEVICE_NAME);
            Toast.makeText(getApplicationContext(), "Connected to " + deviceName, Toast.LENGTH_SHORT).show();
            redrawMenu();
            break;
        case MESSAGE_TOAST:
            Toast.makeText(getApplicationContext(), msg.getData().getInt(TOAST), Toast.LENGTH_SHORT).show();
            break;
        }
    }

    @Override
    public synchronized void onResume() {
        super.onResume();
        if(D) Log.i(TAG, "+ ON RESUME +");

        // Performing this check in onResume() covers the case in which BT was not enabled during onStart(),
        // so we were paused to enable it.
        // onResume() will be called when ACTION_REQUEST_ENABLE activity returns.

        if (mPrintService != null) {
            // Only if the state is STATE_NONE, do we know that we haven't started already
            // then start the bluetooth print services
            if (mPrintService.getState() == BluetoothPrintService.STATE_NONE) {
                mPrintService.start();
            }
        }
    }

    @Override
    public void onDestroy() {
        if(D) Log.i(TAG, "--- ON DESTROY ---");
        // Stop the Bluetooth print services
        if (mPrintService != null) mPrintService.stop();
        super.onDestroy();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_main, menu);
        menu.findItem(R.id.disconnect).setVisible(false);
        mMenu = menu;
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        Intent intent;
        int permissionCheck;

        switch (item.getItemId()) {
        case R.id.secure_connect_scan:
            permissionCheck = ContextCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION);
            if (permissionCheck == PackageManager.PERMISSION_GRANTED) {
                // Launch the DeviceListActivity to see devices and do scan
                intent = new Intent(this, DeviceList.class);
                startActivityForResult(intent, REQUEST_CONNECT_DEVICE_SECURE);
            } else {
                ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.ACCESS_FINE_LOCATION}, PERMISSION_DEVICE_SCAN_SECURE);
            }
            return true;
        case R.id.insecure_connect_scan:
            permissionCheck = ContextCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION);
            if (permissionCheck == PackageManager.PERMISSION_GRANTED) {
                // Launch the DeviceListActivity to see devices and do scan
                intent = new Intent(this, DeviceList.class);
                startActivityForResult(intent, REQUEST_CONNECT_DEVICE_INSECURE);
            } else {
                ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.ACCESS_FINE_LOCATION}, PERMISSION_DEVICE_SCAN_INSECURE);
            }
            return true;
        case R.id.disconnect:
            // Close Bluetooth connection
            if (mPrintService != null)
                mPrintService.start();
            redrawMenu();
            return true;
        }
        return false;
    }

    private void redrawMenu() {
        MenuItem itemSecureConnect = mMenu.findItem(R.id.secure_connect_scan);
        MenuItem itemInsecureConnect = mMenu.findItem(R.id.insecure_connect_scan);
        MenuItem itemDisconnect = mMenu.findItem(R.id.disconnect);

        // Context sensitive option menu
        if (mPrintService.getState() != BluetoothPrintService.STATE_CONNECTED) {
            if (!itemSecureConnect.isVisible()) itemSecureConnect.setVisible(true);
            if (!itemInsecureConnect.isVisible()) itemInsecureConnect.setVisible(true);
            if (itemDisconnect.isVisible()) itemDisconnect.setVisible(false);
        } else {
            if (itemSecureConnect.isVisible()) itemSecureConnect.setVisible(false);
            if (itemInsecureConnect.isVisible()) itemInsecureConnect.setVisible(false);
            if (!itemDisconnect.isVisible()) itemDisconnect.setVisible(true);
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        switch (requestCode) {
            case PERMISSION_DEVICE_SCAN_SECURE:
                // If request is cancelled, the result arrays are empty.
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    Intent intent = new Intent(this, DeviceList.class);
                    startActivityForResult(intent, REQUEST_CONNECT_DEVICE_SECURE);
                }
                break;
            case PERMISSION_DEVICE_SCAN_INSECURE:
                // If request is cancelled, the result arrays are empty.
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    Intent intent = new Intent(this, DeviceList.class);
                    startActivityForResult(intent, REQUEST_CONNECT_DEVICE_INSECURE);
                }
                break;
        }
    }

    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        if(D) Log.d(TAG, "onActivityResult: " + resultCode);

        switch (requestCode) {
        case REQUEST_CONNECT_DEVICE_SECURE:
            if (resultCode == Activity.RESULT_OK) {
                connectDevice(data, true);
            }
            break;
        case REQUEST_CONNECT_DEVICE_INSECURE:
            if (resultCode == Activity.RESULT_OK) {
                connectDevice(data, false);
            }
            break;
        case REQUEST_ENABLE_BT:
            // When the request to enable Bluetooth returns
            if (resultCode == Activity.RESULT_OK) {
                // Bluetooth is now enabled, so set up bluetooth connections
                mPrintService = new BluetoothPrintService(mHandler);
            } else {
                // User did not enable Bluetooth or an error occurred
                if(D) Log.e(TAG, "BT is not enabled");
                Toast.makeText(this, R.string.bt_not_enabled_leaving, Toast.LENGTH_SHORT).show();
                finish();
            }
            break;
        }
    }

    private void connectDevice(Intent data, boolean secure) {
        String address = null;
        // Get the device MAC address
        if (data.getExtras() != null)
            address = data.getExtras().getString(DeviceList.EXTRA_DEVICE_ADDRESS);
        // Get the BLuetoothDevice object
        BluetoothDevice device = mBluetoothAdapter.getRemoteDevice(address);
        // Attempt to connect to the device
        mPrintService.connect(device, secure);
    }
}
