package com.woosim.wifiprint;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.RadioButton;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import com.woosim.printer.WoosimBarcode;
import com.woosim.printer.WoosimCmd;
import com.woosim.printer.WoosimImage;
import com.woosim.printer.WoosimService;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.lang.ref.WeakReference;

public class MainActivity extends AppCompatActivity {
    // Debugging
    private static final String TAG = "MainActivity";
    private static final boolean D = true;

    // Message types sent from the BluetoothPrintService Handler
    public static final int MESSAGE_DEVICE = 1;
    public static final int MESSAGE_TOAST = 2;
    public static final int MESSAGE_READ = 3;

    // Key names received from the BluetoothPrintService Handler
    public static final String DEVICE = "device";
    public static final String TOAST = "toast";

    // Layout Views
    private boolean mEmphasis = false;
    private boolean mUnderline = false;
    private int mCharsize = 1;
    private int mJustification = WoosimCmd.ALIGN_LEFT;
    private TextView mTrack1View;
    private TextView mTrack2View;
    private TextView mTrack3View;
    private Menu mMenu = null;

    // Member object for the print services
    private WifiPrintService mPrintService = null;
    private WoosimService mWoosim = null;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        if(D) Log.i(TAG, "+++ ON CREATE +++");
    }

    @Override
    public void onStart() {
        super.onStart();
        if(D) Log.i(TAG, "++ ON START ++");
        if (mPrintService == null) setupPrint();
    }

    @Override
    public synchronized void onResume() {
        super.onResume();
        if(D) Log.i(TAG, "+ ON RESUME +");
    }

    private void setupPrint() {
        Spinner spinner = findViewById(R.id.spn_charsize);
        ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(
                this, R.array.char_size_array, android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        if (spinner != null) {
            spinner.setAdapter(adapter);
            spinner.setOnItemSelectedListener(
                new AdapterView.OnItemSelectedListener() {
                    public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                        if (position == 1) mCharsize = 2;
                        else if (position == 2) mCharsize = 3;
                        else if (position == 3) mCharsize = 4;
                        else if (position == 4) mCharsize = 5;
                        else if (position == 5) mCharsize = 6;
                        else if (position == 6) mCharsize = 7;
                        else if (position == 7) mCharsize = 8;
                        else mCharsize = 1;
                    }
                    public void onNothingSelected(AdapterView<?> parent) { }
                }
            );
        }
        mTrack1View = findViewById(R.id.view_track1);
        mTrack2View = findViewById(R.id.view_track2);
        mTrack3View = findViewById(R.id.view_track3);

        mPrintService = new WifiPrintService(mHandler);
        mWoosim = new WoosimService(mHandler);
    }

    // The Handler that gets information back from the BluetoothPrintService
    private final MyHandler mHandler = new MyHandler(this);

    private static class MyHandler extends Handler {
        private final WeakReference<MainActivity> mActivity;

        MyHandler(MainActivity activity) {
            mActivity = new WeakReference<>(activity);
        }

        @Override
        public void handleMessage(Message msg) {
            MainActivity activity = mActivity.get();
            if (activity != null) {
                activity.handleMessage(msg);
            }
        }
    }

    private void handleMessage(Message msg) {
        switch (msg.what) {
        case MESSAGE_DEVICE:
            Toast.makeText(getApplicationContext(), msg.getData().getInt(DEVICE), Toast.LENGTH_SHORT).show();
            redrawMenu();
            break;
        case MESSAGE_TOAST:
            Toast.makeText(getApplicationContext(), msg.getData().getInt(TOAST), Toast.LENGTH_SHORT).show();
            break;
        case MESSAGE_READ:
            mWoosim.processRcvData((byte[])msg.obj, msg.arg1);
            break;
        case WoosimService.MESSAGE_PRINTER:
            if (msg.arg1 == WoosimService.MSR) {
                Log.d(TAG, "MSR");
                if (msg.arg2 == 0) {
                    Toast.makeText(getApplicationContext(), "MSR reading failure", Toast.LENGTH_SHORT).show();
                } else {
                    byte[][] track = (byte[][]) msg.obj;
                    if (track[0] != null) {
                        String str = new String(track[0]);
                        mTrack1View.setText(str);
                    }
                    if (track[1] != null) {
                        String str = new String(track[1]);
                        mTrack2View.setText(str);
                    }
                    if (track[2] != null) {
                        String str = new String(track[2]);
                        mTrack3View.setText(str);
                    }
                }
            }
            break;
        }
    }

    @Override
    public synchronized void onPause() {
        super.onPause();
        if(D) Log.i(TAG, "- ON PAUSE -");
    }

    @Override
    public void onStop() {
        super.onStop();
        if(D) Log.i(TAG, "-- ON STOP --");
    }

    @Override
    public void onDestroy() {
        if(D) Log.i(TAG, "--- ON DESTROY ---");
        // Stop the print services
        if (mPrintService != null) mPrintService.stop();
        super.onDestroy();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.activity_main, menu);
        menu.findItem(R.id.disconnect).setVisible(false);
        mMenu = menu;
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        case R.id.wifi_connect:
            final LinearLayout dlg = (LinearLayout) View.inflate(this, R.layout.dlg_connect, null);
            new AlertDialog.Builder(this)
                    .setTitle(R.string.dlg_title)
                    .setView(dlg)
                    .setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int whichButton) {
                            EditText editIp = dlg.findViewById(R.id.ip);
                            String host = editIp.getText().toString();
                            EditText editPort = dlg.findViewById(R.id.port);
                            String port = editPort.getText().toString();
                            mPrintService.connect(host, port);
                        }
                    })
                    .setNegativeButton(R.string.cancel, null)
                    .show();
            return true;
        case R.id.disconnect:
            // Close connection
            if (mPrintService != null) mPrintService.stop();
            redrawMenu();
            return true;
        }
        return false;
    }

    private void redrawMenu() {
        MenuItem itemConnect = mMenu.findItem(R.id.wifi_connect);
        MenuItem itemDisconnect = mMenu.findItem(R.id.disconnect);

        // Context sensitive option menu
        if (mPrintService.getState() != WifiPrintService.STATE_CONNECTED) {
            if (!itemConnect.isVisible()) itemConnect.setVisible(true);
            if (itemDisconnect.isVisible()) itemDisconnect.setVisible(false);
        } else {
            if (itemConnect.isVisible()) itemConnect.setVisible(false);
            if (!itemDisconnect.isVisible()) itemDisconnect.setVisible(true);
        }
    }

    private void sendData(byte[] data) {
        // Check that we're actually connected before trying printing
        if (mPrintService.getState() != WifiPrintService.STATE_CONNECTED) {
            Toast.makeText(this, R.string.not_connected, Toast.LENGTH_SHORT).show();
            return;
        }
        // Check that there's actually something to send
        if (data.length > 0)
            mPrintService.write(data);
    }

    /**
     * On click function for sample print button.
     */
    public void printReceipt(View v) {
        InputStream inStream = getResources().openRawResource(R.raw.receipt2);
        sendData(WoosimCmd.initPrinter());
        try {
            byte[] data = new byte[inStream.available()];
            while (inStream.read(data) != -1)
            {
                sendData(data);
            }
        } catch (IOException e) {
            Log.e(TAG, "sample receipt print fail.", e);
        } finally {
            try {
                inStream.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public void printImage(View v) {
        BitmapFactory.Options options = new BitmapFactory.Options();
        options.inScaled = false;
        Bitmap bmp = BitmapFactory.decodeResource(getResources(), R.drawable.logo, options);
        if (bmp == null) {
            Log.e(TAG, "resource decoding is failed");
            return;
        }
        byte[] data = WoosimImage.printBitmap(0, 0, 384, 200, bmp);
        bmp.recycle();

        sendData(WoosimCmd.setPageMode());
        sendData(data);
        sendData(WoosimCmd.PM_setStdMode());
    }

    public void printText(View v) throws IOException {
        EditText editText = findViewById(R.id.edit_text);
        String string = editText != null ? editText.getText().toString() : null;
        byte[] text = null;

        if (string == null)
            return;
        else {
            try {
                text = string.getBytes("US-ASCII");
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
        }
        ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
        byteStream.write(WoosimCmd.setTextStyle(mEmphasis, mUnderline, false, mCharsize, mCharsize));
        byteStream.write(WoosimCmd.setTextAlign(mJustification));
        if (text != null) byteStream.write(text);
        byteStream.write(WoosimCmd.printData());

        sendData(WoosimCmd.initPrinter());
        sendData(byteStream.toByteArray());
    }

    public void onCheckboxClicked(View v) {
        boolean checked = ((CheckBox) v).isChecked();
        switch(v.getId()) {
        case R.id.cbx_emphasis:
            mEmphasis = checked;
            break;
        case R.id.cbx_underline:
            mUnderline = checked;
            break;
        }
    }

    public void onRadioButtonClicked(View v) {
        boolean checked = ((RadioButton) v).isChecked();
        switch(v.getId()) {
        case R.id.rbtn_left:
            if (checked) mJustification = WoosimCmd.ALIGN_LEFT;
            break;
        case R.id.rbtn_center:
            if (checked) mJustification = WoosimCmd.ALIGN_CENTER;
            break;
        case R.id.rbtn_right:
            if (checked) mJustification = WoosimCmd.ALIGN_RIGHT;
            break;
        }
    }

    /**
     * On click function for barcode print button.
     */
    public void print1DBarcode(View v) throws IOException {
        final byte[] barcode =  {0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x30};
        final byte[] barcode8 = {0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37};
        final byte[] barcodeUPCE = {0x30,0x36,0x35,0x31,0x30,0x30,0x30,0x30,0x34,0x33,0x32,0x37};
        final byte[] cmd_print = WoosimCmd.printData();
        final String title1 = "UPC-A Barcode\r\n";
        byte[] UPCA = WoosimBarcode.createBarcode(WoosimBarcode.UPC_A, 2, 60, true, barcode);
        final String title2 = "UPC-E Barcode\r\n";
        byte[] UPCE = WoosimBarcode.createBarcode(WoosimBarcode.UPC_E, 2, 60, true, barcodeUPCE);
        final String title3 = "EAN13 Barcode\r\n";
        byte[] EAN13 = WoosimBarcode.createBarcode(WoosimBarcode.EAN13, 2, 60, true, barcodeUPCE);
        final String title4 = "EAN8 Barcode\r\n";
        byte[] EAN8 = WoosimBarcode.createBarcode(WoosimBarcode.EAN8, 2, 60, true, barcode8);
        final String title5 = "CODE39 Barcode\r\n";
        byte[] CODE39 = WoosimBarcode.createBarcode(WoosimBarcode.CODE39, 2, 60, true, barcode);
        final String title6 = "ITF Barcode\r\n";
        byte[] ITF = WoosimBarcode.createBarcode(WoosimBarcode.ITF, 2, 60, true, barcode);
        final String title7 = "CODEBAR Barcode\r\n";
        byte[] CODEBAR = WoosimBarcode.createBarcode(WoosimBarcode.CODEBAR, 2, 60, true, barcode);
        final String title8 = "CODE93 Barcode\r\n";
        byte[] CODE93 = WoosimBarcode.createBarcode(WoosimBarcode.CODE93, 2, 60, true, barcode);
        final String title9 = "CODE128 Barcode\r\n";
        byte[] CODE128 = WoosimBarcode.createBarcode(WoosimBarcode.CODE128, 2, 60, true, barcode);

        ByteArrayOutputStream byteStream = new ByteArrayOutputStream(512);
        byteStream.write(title1.getBytes()); byteStream.write(UPCA); byteStream.write(cmd_print);
        byteStream.write(title2.getBytes()); byteStream.write(UPCE); byteStream.write(cmd_print);
        byteStream.write(title3.getBytes()); byteStream.write(EAN13); byteStream.write(cmd_print);
        byteStream.write(title4.getBytes()); byteStream.write(EAN8); byteStream.write(cmd_print);
        byteStream.write(title5.getBytes()); byteStream.write(CODE39); byteStream.write(cmd_print);
        byteStream.write(title6.getBytes()); byteStream.write(ITF); byteStream.write(cmd_print);
        byteStream.write(title7.getBytes()); byteStream.write(CODEBAR); byteStream.write(cmd_print);
        byteStream.write(title8.getBytes()); byteStream.write(CODE93); byteStream.write(cmd_print);
        byteStream.write(title9.getBytes()); byteStream.write(CODE128); byteStream.write(cmd_print);

        sendData(WoosimCmd.initPrinter());
        sendData(byteStream.toByteArray());
    }

    public void print2DBarcode(View v) throws IOException {
        final byte[] barcode = {0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x30};
        final byte[] cmd_print = WoosimCmd.printData();
        final String title1 = "PDF417 2D Barcode\r\n";
        byte[] PDF417 = WoosimBarcode.create2DBarcodePDF417(2, 3, 4, 2, false, barcode);
        final String title2 = "DATAMATRIX 2D Barcode\r\n";
        byte[] dataMatrix = WoosimBarcode.create2DBarcodeDataMatrix(0, 0, 6, barcode);
        final String title3 = "QR-CODE 2D Barcode\r\n";
        byte[] QRCode = WoosimBarcode.create2DBarcodeQRCode(0, (byte)0x4d, 5, barcode);
        final String title4 = "Micro PDF417 2D Barcode\r\n";
        byte[] microPDF417 = WoosimBarcode.create2DBarcodeMicroPDF417(2, 2, 0, 2, barcode);
        final String title5 = "Truncated PDF417 2D Barcode\r\n";
        byte[] truncPDF417 = WoosimBarcode.create2DBarcodeTruncPDF417(2, 3, 4, 2, false, barcode);
        // Maxicode can be printed only with RX version
        final String title6 = "Maxicode 2D Barcode\r\n";
        final byte[] mxcode = {0x41,0x42,0x43,0x44,0x45,0x31,0x32,0x33,0x34,0x35,0x61,0x62,0x63,0x64,0x65};
        byte[] maxCode = WoosimBarcode.create2DBarcodeMaxicode(4, mxcode);

        ByteArrayOutputStream byteStream = new ByteArrayOutputStream(512);
        byteStream.write(title1.getBytes()); byteStream.write(PDF417); byteStream.write(cmd_print);
        byteStream.write(title2.getBytes()); byteStream.write(dataMatrix); byteStream.write(cmd_print);
        byteStream.write(title3.getBytes()); byteStream.write(QRCode); byteStream.write(cmd_print);
        byteStream.write(title4.getBytes()); byteStream.write(microPDF417); byteStream.write(cmd_print);
        byteStream.write(title5.getBytes()); byteStream.write(truncPDF417); byteStream.write(cmd_print);
        byteStream.write(title6.getBytes()); byteStream.write(maxCode); byteStream.write(cmd_print);

        sendData(WoosimCmd.initPrinter());
        sendData(byteStream.toByteArray());
    }

    public void printGS1Databar(View v) throws IOException {
        final byte[] data = {0x30,0x30,0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x30};
        final byte[] cmd_print = WoosimCmd.printData();
        final String title0 = "GS1 Databar type0\r\n";
        byte[] gs0 = WoosimBarcode.createGS1Databar(0, 2, data);
        final String title1 = "GS1 Databar type1\r\n";
        byte[] gs1 = WoosimBarcode.createGS1Databar(1, 2, data);
        final String title2 = "GS1 Databar type2\r\n";
        byte[] gs2 = WoosimBarcode.createGS1Databar(2, 2, data);
        final String title3 = "GS1 Databar type3\r\n";
        byte[] gs3 = WoosimBarcode.createGS1Databar(3, 2, data);
        final String title4 = "GS1 Databar type4\r\n";
        byte[] gs4 = WoosimBarcode.createGS1Databar(4, 2, data);
        final String title5 = "GS1 Databar type5\r\n";
        final byte[] data5 = {0x5b,0x30,0x31,0x5d,0x39,0x30,0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x30,0x38,
                0x5b,0x33,0x31,0x30,0x33,0x5d,0x30,0x31,0x32,0x32,0x33,0x33};
        byte[] gs5 = WoosimBarcode.createGS1Databar(5, 2, data5);
        final String title6 = "GS1 Databar type6\r\n";
        final byte[] data6 = {0x5b,0x30,0x31,0x5d,0x39,0x30,0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x30,0x38,
                0x5b,0x33,0x31,0x30,0x33,0x5d,0x30,0x31,0x32,0x32,0x33,0x33,
                0x5b,0x31,0x35,0x5d,0x39,0x39,0x31,0x32,0x33,0x31};
        byte[] gs6 = WoosimBarcode.createGS1Databar(6, 4, data6);

        ByteArrayOutputStream byteStream = new ByteArrayOutputStream(512);
        byteStream.write(title0.getBytes()); byteStream.write(gs0); byteStream.write(cmd_print);
        byteStream.write(title1.getBytes()); byteStream.write(gs1); byteStream.write(cmd_print);
        byteStream.write(title2.getBytes()); byteStream.write(gs2); byteStream.write(cmd_print);
        byteStream.write(title3.getBytes()); byteStream.write(gs3); byteStream.write(cmd_print);
        byteStream.write(title4.getBytes()); byteStream.write(gs4); byteStream.write(cmd_print);
        byteStream.write(title5.getBytes()); byteStream.write(gs5); byteStream.write(cmd_print);
        byteStream.write(title6.getBytes()); byteStream.write(gs6); byteStream.write(cmd_print);

        sendData(WoosimCmd.initPrinter());
        sendData(byteStream.toByteArray());
    }

    public void setMSRDoubleTrackMode(View v) {
        clearMSRInfo();
        sendData(WoosimCmd.MSR_doubleTrackMode());
        mWoosim.clearRcvBuffer();
    }

    public void setMSRTripleTrackMode(View v) {
        clearMSRInfo();
        sendData(WoosimCmd.MSR_tripleTrackMode());
        mWoosim.clearRcvBuffer();
    }

    public void cancelMSRMode(View v) {
        sendData(WoosimCmd.MSR_exit());
    }

    public void clearMSRInfo(View v) {
        clearMSRInfo();
    }

    private void clearMSRInfo() {
        mTrack1View.setText("");
        mTrack2View.setText("");
        mTrack3View.setText("");
    }
}
