package com.woosim.wifiprint;

import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.SocketAddress;
import java.net.UnknownHostException;
import java.util.Arrays;

/**
 * This class does all the work for setting up and managing TCP socket connections with printers.
 * It has a thread that listens for incoming data and a thread for performing data transmissions when connected.
 */
class WifiPrintService {
    // Debugging
    private static final String TAG = "WifiPrintService";
    private static final boolean D = true;

    // Member fields
    private final Handler mHandler;
    private int mState;
    private ConnectThread mConnectThread;
    private ConnectedThread mConnectedThread;

    // Constants that indicate the current connection state
    private static final int STATE_NONE = 0;       // we're doing nothing
    private static final int STATE_CONNECTING = 1; // now initiating an outgoing connection
    static final int STATE_CONNECTED = 2;          // now connected to a remote device


    /**
     * Constructor.
     * @param handler  A Handler to send messages back to the UI Activity
     */
    WifiPrintService(Handler handler) {
        mState = STATE_NONE;
        mHandler = handler;
    }

    private synchronized void setState(int state) {
        if(D) Log.d(TAG, "setState() " + mState + " -> " + state);
        mState = state;
    }

    synchronized int getState() {
        return mState;
    }

    /**
     * Stop all threads
     */
    synchronized void stop() {
        if(D) Log.d(TAG, "stop");
        if (mConnectThread != null) {
            mConnectThread.cancel();
            mConnectThread = null;
        }
        if (mConnectedThread != null) {
            mConnectedThread.cancel();
            mConnectedThread = null;
        }

        setState(STATE_NONE);
    }

    /**
     * Initiate a connection to a remote device.
     */
    synchronized void connect(String ip_addr, String port) {
        // Cancel any thread attempting to make a connection
        if (mState == STATE_CONNECTING) {
            if (mConnectThread != null) {
                mConnectThread.cancel();
                mConnectThread = null;
            }
        }
        // Cancel any thread currently running a connection
        if (mConnectedThread != null) {
            mConnectedThread.cancel();
            mConnectedThread = null;
        }
        // Start the thread to connect with the given device
        mConnectThread = new ConnectThread(ip_addr, port);
        mConnectThread.start();

        setState(STATE_CONNECTING);
    }

    private synchronized void connected(Socket socket) {
        // Cancel the thread that completed the connection
        if (mConnectThread != null) {
            mConnectThread.cancel();
            mConnectThread = null;
        }
        // Cancel any thread currently running a connection
        if (mConnectedThread != null) {
            mConnectedThread.cancel();
            mConnectedThread = null;
        }
        // Start the thread to manage the connection and perform transmissions
        mConnectedThread = new ConnectedThread(socket);
        mConnectedThread.start();
        // Send the name of the connected device back to the UI Activity
        Message msg = mHandler.obtainMessage(MainActivity.MESSAGE_DEVICE);
        Bundle bundle = new Bundle();
        bundle.putInt(MainActivity.DEVICE, R.string.msg_connect);
        msg.setData(bundle);
        mHandler.sendMessage(msg);

        setState(STATE_CONNECTED);
    }

    /**
     * Write to the ConnectedThread in an unsynchronized manner
     * Using thread to avoid NetworkOnMainThreadException from Android Nougat
     * @param out The bytes to write
     */
    void write(final byte[] out) {
        new Thread(new Runnable() {
            @Override
            public void run() {
                mConnectedThread.write(out);
            }
        }).start();
    }

    /**
     * Indicate that the connection attempt failed and notify the UI Activity.
     */
    private void connectionFailed(int str_id) {
        // When the application is destroyed, just return
        if (mState == STATE_NONE) return;
        // Send a failure message back to the Activity
        Message msg = mHandler.obtainMessage(MainActivity.MESSAGE_DEVICE);
        Bundle bundle = new Bundle();
        bundle.putInt(MainActivity.DEVICE, str_id);
        msg.setData(bundle);
        mHandler.sendMessage(msg);

        WifiPrintService.this.stop();
    }

    /**
     * This thread runs while attempting to make an outgoing connection with a device.
     * It runs straight through the connection either succeeds or fails.
     */
    private class ConnectThread extends Thread {
        private final Socket mmSocket;
        private final String mmDeviceIP;
        private final int mmPort;

        ConnectThread(String ip_addr, String port) {
            mmSocket = new Socket();
            mmDeviceIP = ip_addr;
            mmPort = Integer.parseInt(port);
        }

        public void run() {
            if(D) Log.i(TAG, "begin mConnectThread");
            // Make a connection to the Socket
            try {
                InetAddress serverAddr = InetAddress.getByName(mmDeviceIP);
                SocketAddress socketAddress = new InetSocketAddress(serverAddr, mmPort);
                mmSocket.connect(socketAddress, 2000);
            } catch (UnknownHostException e) {
                Log.e(TAG, "Device IP is unknown host", e);
                try {
                    mmSocket.close();
                } catch (IOException e1) {
                    Log.e(TAG, "unable to close socket during connection failure");
                    e1.printStackTrace();
                }
                connectionFailed(R.string.msg_unknown_ip);
                return;
            } catch (IOException e) {
                Log.e(TAG, "socket connection is failed", e);
                try {
                    mmSocket.close();
                } catch (IOException e1) {
                    Log.e(TAG, "unable to close socket during connection failure");
                    e1.printStackTrace();
                }
                connectionFailed(R.string.msg_connect_fail);
                return;
            }
            // Reset the ConnectThread because we're done
            synchronized (WifiPrintService.this) {
                mConnectThread = null;
            }
            // Start the connected thread
            connected(mmSocket);
        }

        void cancel() {
            try {
                mmSocket.close();
            } catch (IOException e) {
                Log.e(TAG, "unable to close socket during connection failure");
                e.printStackTrace();
            }
        }
    }

    /**
     * This thread runs after a connection with a remote device.
     * It handles all incoming and outgoing transmissions.
     */
    private class ConnectedThread extends Thread {
        private final Socket mmSocket;
        private final InputStream mmInStream;
        private final OutputStream mmOutStream;

        ConnectedThread(Socket socket) {
            if(D) Log.d(TAG, "create ConnectedThread");
            mmSocket = socket;
            InputStream tmpIn = null;
            OutputStream tmpOut = null;
            // Get the input and output streams
            try {
                tmpIn = socket.getInputStream();
                tmpOut = socket.getOutputStream();
            } catch (IOException e) {
                Log.e(TAG, "temp sockets not created", e);
            }
            mmInStream = tmpIn;
            mmOutStream = tmpOut;
        }

        public void run() {
            if(D) Log.i(TAG, "BEGIN mConnectedThread");
            byte[] buffer = new byte[1024];
            int bytes;
            // Keep listening to the InputStream while connected
            while (true) {
                try {
                    // Read from the InputStream
                    bytes = mmInStream.read(buffer);
                    // buffer can be over-written by next input stream data, so it should be copied
                    byte[] rcvData = Arrays.copyOf(buffer, bytes);
                    // Send the obtained bytes to the UI Activity
                    mHandler.obtainMessage(MainActivity.MESSAGE_READ, bytes, -1, rcvData).sendToTarget();
                } catch (IOException e) {
                    Log.e(TAG, "disconnected", e);
                    connectionFailed(R.string.msg_connect_lost);
                    break;
                }
            }
        }

        /**
         * Write to the connected OutStream.
         * @param buffer  The bytes to write
         */
        synchronized void write(byte[] buffer) {
            try {
                mmOutStream.write(buffer);
            } catch (IOException e) {
                Log.e(TAG, "Exception during write", e);
            }
        }

        void cancel() {
            try {
                mmInStream.close();
                mmOutStream.close();
                mmSocket.close();
            } catch (IOException e) {
                Log.e(TAG, "unable to close stream or socket during disconnection", e);
            }
        }
    }
}
